import oommfc as oc
import micromagneticmodel.tests as mmt


class TestDemag(mmt.TestDemag):
    def test_script(self):
        demag = oc.Demag()

        script = demag.script()
        assert script.count("\n") == 3
        assert script[0] == "#"
        assert script[-1] == "\n"

        lines = script.split("\n")
        assert len(lines) == 4
        assert lines[0] == "# Demag"
        assert lines[1] == "Specify Oxs_Demag {}"



import oommfc as oc


class TestHamiltonian:
    def setup(self):
        A = 1e-12
        self.exchange = oc.Exchange(A)
        H = (0, 0, 1.2e6)
        self.zeeman = oc.Zeeman(H)
        K = 1e4
        u = (0, 1, 0)
        self.uniaxialanisotropy = oc.UniaxialAnisotropy(K, u)
        self.demag = oc.Demag()

        self.terms = [self.exchange,
                      self.zeeman,
                      self.uniaxialanisotropy,
                      self.demag]

    def test_script(self):
        hamiltonian = oc.Hamiltonian()
        for term in self.terms:
            hamiltonian.add(term)

        script = hamiltonian.script()

        assert script.count("#") == 4
        assert script.count("Specify") == 4
        assert script[0] == "#"
        assert script[-1] == "\n"

        assert "UniformExchange" in script
        assert "FixedZeeman" in script
        assert "UniaxialAnisotropy" in script
        assert "Demag" in script

        assert "A" in script
        assert "K1" in script
        assert "axis" in script
        assert "multiplier" in script
        assert "vector" in script
        assert "field" in script



import os
import discretisedfield as df
import oommfc as oc


class TestDriver:
    def setup(self):
        self.system = oc.System(name='tds')
        mesh = oc.Mesh((0, 0, 0),
                       (100e-9, 100e-9, 10e-9),
                       (10e-9, 10e-9, 10e-9))
        self.system.mesh = mesh
        self.system.hamiltonian += oc.Exchange(1.5e-11)
        self.system.hamiltonian += oc.Demag()
        self.system.dynamics += oc.Precession(2.211e5)
        self.system.dynamics += oc.Damping(0.02)
        self.system.m = df.Field(mesh, value=(0, 1, 0),
                                 normalisedto=8e5)

    def test_makedir(self):
        driver = oc.Driver()
        driver._makedir(self.system)

        dirname = "{}/".format(self.system.name)
        assert os.path.exists(dirname)

        os.system("rm -r {}".format(dirname))

    def test_filenames(self):
        driver = oc.Driver()
        filenames = driver._filenames(self.system)

        assert len(filenames.keys()) == 3
        assert filenames["dirname"] == 'tds/'
        assert filenames["omffilename"] == 'tds/m0.omf'
        assert filenames["miffilename"] == 'tds/tds.mif'



from .driver import Driver


class MinDriver(Driver):
    def script(self, system):
        meshname = system.mesh.name
        Ms = system.m.normalisedto
        systemname = system.name

        mif = "# CGEvolver\n"
        mif += "Specify Oxs_CGEvolve {}\n\n"
        mif += "# MinDriver\n"
        mif += "Specify Oxs_MinDriver {\n"
        mif += "  evolver Oxs_CGEvolve\n"
        mif += "  stopping_mxHxm 0.01\n"
        mif += "  mesh :{}\n".format(meshname)
        mif += "  Ms {}\n".format(Ms)
        mif += "  m0 {\n"
        mif += "    Oxs_FileVectorField {\n"
        mif += "      atlas :atlas\n"
        mif += "      norm 1.0\n"
        mif += "      file m0.omf\n"
        mif += "    }\n"
        mif += "  }\n"
        mif += "  basename {}\n".format(systemname)
        mif += "  vector_field_output_format {text %\#.8g}\n"
        mif += "}\n\n"
        mif += "Destination table mmArchive\n"
        mif += "Destination mags mmArchive\n\n"
        mif += "Schedule DataTable table Stage 1\n"
        mif += "Schedule Oxs_MinDriver::Spin mags Stage 1"

        return mif

    def _check_args(self, **kwargs):
        pass



import textwrap
import discretisedfield as df


class Mesh(df.Mesh):
    def script(self):
        mif = "# BoxAtlas\n"
        mif += "Specify Oxs_BoxAtlas:atlas {\n"
        mif += "  xrange {{{} {}}}\n".format(self.p1[0], self.p2[0])
        mif += "  yrange {{{} {}}}\n".format(self.p1[1], self.p2[1])
        mif += "  zrange {{{} {}}}\n".format(self.p1[2], self.p2[2])
        mif += "}\n\n"
        mif += "# RectangularMesh\n"
        mif += "Specify Oxs_RectangularMesh:{} {{\n".format(self.name)
        mif += "  cellsize {{{} {} {}}}\n".format(self.cell[0],
                                                  self.cell[1],
                                                  self.cell[2])
        mif += "  atlas Oxs_BoxAtlas:atlas\n"
        mif += "}\n\n"

        return mif



import pytest
import oommfc as oc
import micromagneticmodel.tests as mmt


class TestDamping(mmt.TestDamping):
    def test_script(self):
        for alpha in self.valid_args:
            damping = oc.Damping(alpha)
            with pytest.raises(NotImplementedError):
                damping.script()



import micromagneticmodel as mm


class Exchange(mm.Exchange):
    def script(self):
        mif = "# UniformExchange\n"
        mif += "Specify Oxs_UniformExchange {\n"
        mif += "  A {}\n".format(self.A)
        mif += "}\n\n"

        return mif



from .oommf import OOMMF
from .hamiltonian import Exchange, UniaxialAnisotropy, \
    Demag, Zeeman, Hamiltonian
from .dynamics import Precession, Damping, STT, Dynamics
from .drivers import Driver, MinDriver, TimeDriver, HysteresisDriver
from .mesh import Mesh
from .system import System
from micromagneticmodel.consts import mu0, e, me, kB, h, g, \
    hbar, gamma, muB, gamma0


oommf = OOMMF()

def test():
    """Runs all the tests"""
    oommf.status(raise_exception=True)  # pragma: no cover
    import pytest  # pragma: no cover
    args = ["-v", "--pyargs", "oommfc"]  # pragma: no cover
    pytest.main(args)  # pragma: no cover


def test_non_oommf():
    """ Runs tests that do not need an OOMMF installation"""
    import pytest  # pragma: no cover
    args = ["-m", "not oommf", "-v",
            "--pyargs", "oommfc"]  # pragma: no cover
    pytest.main(args)  # pragma: no cover


def test_oommf():
    """Runs all tests that require an OOMMF installation."""
    oommf.status(raise_exception=True)  # pragma: no cover
    import pytest  # pragma: no cover
    args = ["-m", "oommf", "-v",
            "--pyargs", "oommfc"]  # pragma: no cover
    pytest.main(args)  # pragma: no cover



from .driver import Driver


class HysteresisDriver(Driver):
    def script(self, system, **kwargs):
        meshname = system.mesh.name
        Ms = system.m.normalisedto
        systemname = system.name
        Hmin = kwargs["Hmin"]
        Hmax = kwargs["Hmax"]
        n = kwargs["n"]

        mif = "# UZeeman\n"
        mif += "Specify Oxs_UZeeman {\n"
        mif += "  Hrange {\n"
        mif += "    {{ {} {} {} {} {} {} {} }}\n".format(Hmin[0],
                                                         Hmin[1],
                                                         Hmin[2],
                                                         Hmax[0],
                                                         Hmax[1],
                                                         Hmax[2],
                                                         n)
        mif += "    {{ {} {} {} {} {} {} {} }}\n".format(Hmax[0],
                                                         Hmax[1],
                                                         Hmax[2],
                                                         Hmin[0],
                                                         Hmin[1],
                                                         Hmin[2],
                                                         n)
        mif += "  }\n"
        mif += "}\n\n"
        mif += "# CGEvolver\n"
        mif += "Specify Oxs_CGEvolve {}\n\n"
        mif += "# MinDriver\n"
        mif += "Specify Oxs_MinDriver {\n"
        mif += "  evolver Oxs_CGEvolve\n"
        mif += "  stopping_mxHxm 0.01\n"
        mif += "  mesh :{}\n".format(meshname)
        mif += "  Ms {}\n".format(Ms)
        mif += "  m0 {\n"
        mif += "    Oxs_FileVectorField {\n"
        mif += "      atlas :atlas\n"
        mif += "      norm 1.0\n"
        mif += "      file m0.omf\n"
        mif += "    }\n"
        mif += "  }\n"
        mif += "  basename {}\n".format(systemname)
        mif += "  vector_field_output_format {text %\#.8g}\n"
        mif += "}\n\n"
        mif += "Destination table mmArchive\n"
        mif += "Destination mags mmArchive\n\n"
        mif += "Schedule DataTable table Stage 1\n"
        mif += "Schedule Oxs_MinDriver::Spin mags Stage 1"
        
        return mif



import micromagneticmodel as mm


class UniaxialAnisotropy(mm.UniaxialAnisotropy):
    def script(self):
        mif = "# UniaxialAnisotropy\n"
        mif += "Specify Oxs_UniaxialAnisotropy {\n"
        mif += "  K1 {}\n".format(self.K)
        mif += "  axis {{{} {} {}}}\n".format(self.u[0], self.u[1], self.u[2])
        mif += "}\n\n"

        return mif



import os
import sarge


class OOMMF:
    def __init__(self, varname="OOMMFTCL", dockername="docker",
                 dockerimage="joommf/oommf", where=None):
        self.varname = varname
        self.dockername = dockername
        self.dockerimage = dockerimage
        self.statusdict = self.status(raise_exception=False)
        self.where = self._where_to_run(where)

    def status(self, raise_exception=False, verbose=False):
        # OOMMF status on host
        cmd = ("tclsh", os.getenv(self.varname, "wrong"), "boxsi",
               "+fg", "+version", "-exitondone", "1")
        try:
            poommf = sarge.capture_both(cmd)
            returncode = poommf.returncode
        except FileNotFoundError:
            returncode = 1
        if returncode:
            host = False
            if verbose:
                oommfpath = os.getenv(self.varname)
                if oommfpath is None:
                    print("Cannot find {} path.".format(self.varname))
                elif not os.path.isfile(oommfpath):
                    print("{} path {} set to a non-existing "
                          "file.".format(self.varname, oommfpath))
                else:
                    print("{} path {} set to an existing "
                          "file.".format(self.varname, oommfpath))
                    print("Something wrong with OOMMF installation.")
        else:
            host = True

        # Docker status
        cmd = (self.dockername, "images")
        try:
            pdocker = sarge.capture_both(cmd)
            returncode = pdocker.returncode
        except FileNotFoundError:
            returncode = 1

        if returncode:
            docker = False
            if verbose:
                print("Docker not installed/active.")
        else:
            docker = True

        # Raise exception if required
        if not (host or docker) and raise_exception:
            raise EnvironmentError("OOMMF and docker not found.")

        return {"host": host, "docker": docker}

    def call(self, argstr, where=None):
        if where is None:
            where = self.where
        if self.statusdict[where]:
            if where == "host":
                return self._call_host(argstr=argstr)
            elif where == "docker":
                return self._call_docker(argstr=argstr)

    def version(self, where=None):
        where = self._where_to_run(where=where)
        p = self.call(argstr="+version", where=where)
        return p.stderr.text.split("oommf.tcl")[-1].strip()

    def _where_to_run(self, where):
        if where is None:
            if self.statusdict["host"]:
                return "host"
            else:
                return "docker"
        else:
            return where

    def _call_host(self, argstr):
        oommfpath = os.getenv(self.varname, None)
        cmd = ("tclsh", oommfpath, "boxsi", "+fg",
               argstr, "-exitondone", "1")
        return sarge.capture_both(cmd)

    def _call_docker(self, argstr):
        returncode = sarge.capture_both([self.dockername,
                                         "pull",
                                         self.dockerimage])
        cmd = ("{} run -v {}:/io {} /bin/bash -c \"tclsh "
               "/usr/local/oommf/oommf/oommf.tcl boxsi +fg {} "
               "-exitondone 1\"").format(self.dockername, os.getcwd(),
                                         self.dockerimage, argstr)
        return sarge.capture_both(cmd)



import oommfc as oc


class TestSystem:
    def test_script(self):
        system = oc.System(name="test_system")

        system.mesh = oc.Mesh((0, 0, 0), (5, 5, 5), (1, 1, 1))

        system.hamiltonian += oc.Exchange(1e-12)
        system.hamiltonian += oc.Demag()
        system.hamiltonian += oc.UniaxialAnisotropy(1e3, (0, 1, 0))
        system.hamiltonian += oc.Zeeman((0, 1e6, 0))

        system.dynamics += oc.Precession(2.211e5)
        system.dynamics += oc.Damping(0.1)

        system.m = lambda pos: (0, 1, 0)

        script = system.script()

        assert script[0] == "#"
        assert script[-1] == "\n"
        assert script.count("#") == 6
        assert script.count("Specify") == 6
        assert "Exchange" in script
        assert "Demag" in script
        assert "Zeeman" in script
        assert "UniaxialAnisotropy" in script
        assert "BoxAtlas" in script
        assert "RectangularMesh" in script



import pytest
import oommfc as oc
import micromagneticmodel.tests as mmt


class TestDynamics(mmt.TestDynamics):
    def test_script(self):
        dynamics = oc.Dynamics()
        for term in self.terms:
            dynamics += term

            with pytest.raises(NotImplementedError):
                dynamics.script()



import os
import glob
import pytest
import oommfc as oc
from .test_driver import TestDriver


class TestMinDriver(TestDriver):
    def test_script(self):
        md = oc.MinDriver()

        script = md.script(self.system)

        assert script[0] == "#"
        assert script[-1] == "1"
        assert script.count("#") == 3
        assert script.count("Specify") == 2
        assert script.count("Destination") == 2
        assert script.count("Schedule") == 2
        assert script.count("mmArchive") == 2
        assert script.count("Stage") == 2

        assert "Oxs_CGEvolve" in script
        assert "Oxs_MinDriver" in script
        assert "Oxs_FileVectorField" in script

        lines = script.split("\n")
        assert lines[8] == "  Ms {}".format(8e5)
        assert lines[13] == "      file m0.omf"
        assert lines[16] == "  basename tds"

    def test_save_mif(self):
        md = oc.MinDriver()

        md._makedir(self.system)
        md._save_mif(self.system)

        assert os.path.isfile("tds/tds.mif")

        lines = open("tds/tds.mif", "r").readlines()
        assert lines[0] == "# MIF 2.1\n"
        assert lines[-1][-1] == "1"

        os.system("rm -r tds/")

    @pytest.mark.oommf
    def test_drive(self):
        md = oc.MinDriver()

        md.drive(self.system)

        assert os.path.exists("tds/")
        assert os.path.isfile("tds/tds.mif")

        omf_files = list(glob.iglob("tds/*.omf"))
        odt_files = list(glob.iglob("tds/*.odt"))

        assert len(omf_files) == 2
        assert "tds/m0.omf" in omf_files

        assert len(odt_files) == 1

        os.system("rm -r tds/")



import oommfc as oc
import discretisedfield as df

mesh = oc.Mesh((0, 0, 0), (100e-9, 100e-9, 10e-9), (5e-9, 5e-9, 5e-9))
system = oc.System(name="mysystem")

system.mesh = mesh
system.hamiltonian = oc.Exchange(A=1.2e-11) + oc.Zeeman(H=(1e6, 1e6, 0))
system.m = df.Field(mesh, value=(0, 0, 1), normalisedto=8e6)

md = oc.MinDriver()
md.drive(system)

print(system.hamiltonian.exchange.energy())



from .driver import Driver


class TimeDriver(Driver):
    def script(self, system, **kwargs):
        try:
            alpha = system.dynamics.damping.alpha
        except AttributeError:
            alpha = 0
        try:
            gamma = system.dynamics.precession.gamma
        except AttributeError:
            gamma = 2.211e5
        try:
            u = system.dynamics.stt.u
            beta = system.dynamics.stt.beta
        except AttributeError:
            stt = False
        else:
            stt = True

        meshname = system.mesh.name
        Ms = system.m.normalisedto
        systemname = system.name

        if not stt:
            mif = "# RungeKuttaEvolver\n"
            mif += "Specify Oxs_RungeKuttaEvolve:evolver {\n"
            mif += "  alpha {}\n".format(alpha)
            mif += "  gamma_G {}\n".format(gamma)
            mif += "}\n\n"
            evolver = "Oxs_RungeKuttaEvolve"
        else:
            mif = "# SpinTransferTorqueEvolver\n"
            mif += "Specify Anv_SpinTEvolve {\n"
            mif += "  do_precess 1\n"
            mif += "  gamma_G {}\n".format(gamma)
            mif += "  alpha {}\n".format(alpha)
            mif += "  method rkf54s\n"
            mif += "  u {\n"
            mif += "    Oxs_UniformScalarField {\n"
            mif += "      value {}\n".format(u[0])
            mif += "    }\n"
            mif += "  }\n"
            mif += "  beta {}\n".format(beta)
            mif += "}\n\n"
            evolver = "Anv_SpinTEvolve"

        mif += "# TimeDriver\n"
        mif += "Specify Oxs_TimeDriver {\n"
        mif += "  evolver {}\n".format(evolver)
        mif += "  stopping_time {}\n".format(kwargs["t"]/kwargs["n"])
        mif += "  mesh :{}\n".format(meshname)
        mif += "  stage_count {}\n".format(kwargs["n"])
        mif += "  Ms {}\n".format(Ms)
        mif += "  m0 {\n"
        mif += "    Oxs_FileVectorField {\n"
        mif += "      atlas :atlas\n"
        mif += "      norm 1.0\n"
        mif += "      file m0.omf\n"
        mif += "    }\n"
        mif += "  }\n"
        mif += "  basename {}\n".format(systemname)
        mif += "  vector_field_output_format {text %\#.8g}\n"
        mif += "}\n\n"
        mif += "Destination table mmArchive\n"
        mif += "Destination mags mmArchive\n\n"
        mif += "Schedule DataTable table Stage 1\n"
        mif += "Schedule Oxs_TimeDriver::Spin mags Stage 1"

        return mif

    def _check_args(self, **kwargs):
        t, n = kwargs["t"], kwargs["n"]

        if t <= 0:
            raise ValueError("Expected t > 0.")
        if n <= 0 or not isinstance(n, int):
            raise ValueError("Expected n > 0.")



import oommfc as oc
import discretisedfield.tests as dft


class TestMesh(dft.TestMesh):
    def test_get_mif(self):
        for arg in self.valid_args:
            p1 = arg[0]
            p2 = arg[1]
            cell = arg[2]
            name = "test_mesh"

            mesh = oc.Mesh(p1, p2, cell, name=name)

            script = mesh.script()
            assert script.count("\n") == 13
            assert script[0] == "#"
            assert script[-1] == "\n"

            lines = script.split("\n")
            assert len(lines) == 14

            # Assert BoxAtlas script
            assert lines[0] == "# BoxAtlas"
            assert lines[1] == "Specify Oxs_BoxAtlas:atlas {"
            assert lines[2] == "  xrange {{{} {}}}".format(p1[0], p2[0])
            assert lines[3] == "  yrange {{{} {}}}".format(p1[1], p2[1])
            assert lines[4] == "  zrange {{{} {}}}".format(p1[2], p2[2])
            assert lines[5] == "}"

            # Empty line between BoxAtlas and RectangularMesh
            assert lines[6] == ""

            # Assert RectangularMesh script
            assert lines[7] == "# RectangularMesh"
            assert lines[8] == "Specify Oxs_RectangularMesh:{} {{".format(name)
            assert lines[9] == "  cellsize {{{} {} {}}}".format(cell[0],
                                                                cell[1],
                                                                cell[2])
            assert lines[10] == "  atlas Oxs_BoxAtlas:atlas"
            assert lines[11] == "}"



import oommfc as oc
import micromagneticmodel.tests as mmt


class TestUniaxialAnisotropy(mmt.TestUniaxialAnisotropy):
    def test_script(self):
        for arg in self.valid_args:
            K = arg[0]
            u = arg[1]
            name = "anisotropy_test"

            anisotropy = oc.UniaxialAnisotropy(K, u, name=name)

            script = anisotropy.script()
            assert script.count("\n") == 6
            assert script[0] == "#"
            assert script[-1] == "\n"

            lines = script.split("\n")
            assert len(lines) == 7
            assert lines[0] == "# UniaxialAnisotropy"
            assert lines[1] == "Specify Oxs_UniaxialAnisotropy {"
            assert lines[2] == "  K1 {}".format(K)
            assert lines[3] == "  axis {{{} {} {}}}".format(u[0], u[1], u[2])
            assert lines[4] == "}"



import pytest
import oommfc as oc
import micromagneticmodel.tests as mmt


class TestPrecession(mmt.TestPrecession):
    def test_script(self):
        for gamma in self.valid_args:
            precession = oc.Precession(gamma)
            with pytest.raises(NotImplementedError):
                precession.script()



import micromagneticmodel as mm


class System(mm.System):
    def script(self):
        mif = self.mesh.script()
        mif += self.hamiltonian.script()
        return mif

    def total_energy(self):
        for key in self.dt.tail(1).keys():
            if "Totalenergy" in key:
                return self.dt.tail(1)[key][0]



import os
import glob
import oommfc as oc
from .test_driver import TestDriver


class TestHysteresisDriver(TestDriver):
    def test_script(self):
        hd = oc.HysteresisDriver()

        script = hd.script(self.system, Hmin=(0, 0, 0),
                           Hmax=(10, 10, 10), n=10)

        assert script[0] == "#"
        assert script[-1] == "1"
        assert script.count("#") == 4
        assert script.count("Specify") == 3
        assert script.count("Destination") == 2
        assert script.count("Schedule") == 2
        assert script.count("mmArchive") == 2
        assert script.count("Stage") == 2

        assert "Oxs_UZeeman" in script
        assert "Oxs_CGEvolve" in script
        assert "Oxs_MinDriver" in script
        assert "Oxs_FileVectorField" in script



import pytest
import oommfc as oc


class TestOOMMF:
    @pytest.mark.oommf
    def test_status(self):
        # Case 1: host True, docker True
        varname = "OOMMFTCL"
        dockername = "docker"
        raise_exception = True
        oommf = oc.OOMMF(varname=varname,
                         dockername=dockername)
        status = oommf.status(raise_exception=raise_exception,
                              verbose=True)
        assert isinstance(status, dict)
        assert status["host"] is True
        assert status["docker"] is True

        # Case 2: host True, docker False
        varname = "OOMMFTCL"
        dockername = "dockerwrongname"
        raise_exception = True
        oommf = oc.OOMMF(varname=varname,
                         dockername=dockername)
        status = oommf.status(raise_exception=raise_exception,
                              verbose=True)
        assert isinstance(status, dict)
        assert status["host"] is True
        assert status["docker"] is False

        # Case 3: host False (wrong varname), docker True
        varname = "OOMMFWRONGVARNAME"
        dockername = "docker"
        raise_exception = True
        oommf = oc.OOMMF(varname=varname,
                         dockername=dockername)
        status = oommf.status(raise_exception=raise_exception,
                              verbose=True)
        assert isinstance(status, dict)
        assert status["host"] is False
        assert status["docker"] is True

        # Case 4: host False (wrong path), docker True
        varname = "OOMMFWRONGPATH"
        dockername = "docker"
        raise_exception = True
        oommf = oc.OOMMF(varname=varname,
                         dockername=dockername)
        status = oommf.status(raise_exception=raise_exception,
                              verbose=True)
        assert isinstance(status, dict)
        assert status["host"] is False
        assert status["docker"] is True

        # Case 5: host False (wrong path), docker False, no exception
        varname = "WRONGWRONGWRONG"
        dockername = "dockerwrong"
        raise_exception = False
        oommf = oc.OOMMF(varname=varname,
                         dockername=dockername)
        status = oommf.status(raise_exception=raise_exception,
                              verbose=True)
        assert isinstance(status, dict)
        assert status["host"] is False
        assert status["docker"] is False

        # Case 6: host False (wrong path), docker False, raise exception
        varname = "WRONGWRONGWRONG"
        dockername = "dockerwrong"
        raise_exception = True
        with pytest.raises(EnvironmentError):
            oommf = oc.OOMMF(varname=varname,
                             dockername=dockername)
            status = oommf.status(raise_exception=raise_exception,
                                  verbose=True)

        # Case 7: host False (wrong file), docker False, no exception
        varname = "OOMMFWRONGFILE"
        dockername = "dockerwrong"
        raise_exception = False
        oommf = oc.OOMMF(varname=varname,
                         dockername=dockername)
        status = oommf.status(raise_exception=raise_exception,
                              verbose=True)
        assert isinstance(status, dict)
        assert status["host"] is False
        assert status["docker"] is False

    @pytest.mark.oommf
    def test_call_oommf(self):
        argstr = "+version"
        varname = "OOMMFTCL"
        dockername = "docker"
        raise_exception = True
        dockerimage = "joommf/oommf"
        for where in [None, "host", "docker"]:
            oommf = oc.OOMMF(varname=varname,
                             dockername=dockername,
                             dockerimage=dockerimage,
                             where=where)
            oommf.call(argstr=argstr)

    @pytest.mark.oommf
    def test_version(self):
        oommf = oc.OOMMF()
        version = oommf.version(where=None)
        assert isinstance(version, str)
        assert "." in version
        assert version[0].isdigit() and version[-1].isdigit()
        assert 5 < len(version) < 10

        oommf = oc.OOMMF()
        version = oommf.version(where="host")
        assert isinstance(version, str)
        assert "." in version
        assert version[0].isdigit() and version[-1].isdigit()
        assert 5 < len(version) < 10

        oommf = oc.OOMMF()
        version = oommf.version(where="docker")
        assert isinstance(version, str)
        assert "." in version
        assert version[0].isdigit() and version[-1].isdigit()
        assert 5 < len(version) < 10

    @pytest.mark.oommf
    def test_where_to_run(self):
        # Case 1: choose "host", both working
        where = None
        varname = "OOMMFTCL"
        dockername = "docker"
        oommf = oc.OOMMF(varname=varname, dockername=dockername)
        where = oommf._where_to_run(where=where)
        assert where == "host"

        # Case 2: choose "host", only host working
        where = None
        varname = "OOMMFTCL"
        dockername = "wrongdocker"
        oommf = oc.OOMMF(varname=varname, dockername=dockername)
        where = oommf._where_to_run(where=where)
        assert where == "host"

        # Case 2: choose "docker", only docker working
        where = None
        varname = "WRONGOMMFPATH"
        dockername = "docker"
        oommf = oc.OOMMF(varname=varname, dockername=dockername)
        where = oommf._where_to_run(where=where)
        assert where == "docker"



import setuptools

with open('README.rst') as f:
    readme = f.read()

setuptools.setup(
    name='oommfc',
    version='0.5.6.3',
    description='A Python-based OOMMF calculator.',
    long_description=readme,
    url='https://joommf.github.io',
    author='Marijan Beg, Ryan A. Pepper, and Hans Fangohr',
    author_email='jupyteroommf@gmail.com',
    packages=setuptools.find_packages(),
    install_requires=['scipy',
                      'sarge',
                      'discretisedfield',
                      'micromagneticmodel',
                      'oommfodt'],
    classifiers=['Development Status :: 3 - Alpha',
                 'License :: OSI Approved :: BSD License',
                 'Topic :: Scientific/Engineering :: Physics',
                 'Intended Audience :: Science/Research',
                 'Programming Language :: Python :: 3 :: Only']
)



import pytest
import oommfc as oc
import micromagneticmodel.tests as mmt


class TestSTT(mmt.TestSTT):
    def test_script(self):
        for arg in self.valid_args:
            u, beta = arg
            stt = oc.STT(u, beta)
            with pytest.raises(NotImplementedError):
                stt.script()



import os
import glob
import pytest
import oommfc as oc
import discretisedfield as df
import pytest


@pytest.mark.oommf
def test_stdprob4():
    name = "stdprob4"

    # Remove any previous simulation directories.
    os.system("rm -rf {}/".format(name))

    L, d, th = 500e-9, 125e-9, 3e-9   # (m)
    cellsize = (5e-9, 5e-9, 3e-9)  # (m)
    mesh = oc.Mesh((0, 0, 0), (L, d, th), cellsize)

    system = oc.System(name=name)
    system.mesh = mesh

    A = 1.3e-11  # (J/m)
    system.hamiltonian = oc.Exchange(A) + oc.Demag()

    gamma = 2.211e5  # (m/As)
    alpha = 0.02
    system.dynamics = oc.Precession(gamma) + oc.Damping(alpha)

    Ms = 8e5  # (A/m)
    system.m = df.Field(mesh, value=(1, 0.25, 0.1),
                        normalisedto=Ms)

    md = oc.MinDriver()
    md.drive(system)  # updates system.m in-place

    assert os.path.exists("{}/".format(name))
    assert os.path.isfile("{}/{}.mif".format(name, name))

    omf_files = list(glob.iglob("{}/*.omf".format(name)))
    odt_files = list(glob.iglob("{}/*.odt".format(name)))

    assert len(omf_files) == 2
    assert "{}/m0.omf".format(name) in omf_files

    assert len(odt_files) == 1

    os.system("rm -r {}/".format(name))

    H = (-24.6e-3/oc.mu0, 4.3e-3/oc.mu0, 0)
    system.hamiltonian += oc.Zeeman(H)

    td = oc.TimeDriver()
    td.drive(system, t=1e-9, n=200)

    assert os.path.exists("{}/".format(name))
    assert os.path.isfile("{}/{}.mif".format(name, name))

    omf_files = list(glob.iglob("{}/*.omf".format(name)))
    odt_files = list(glob.iglob("{}/*.odt".format(name)))

    assert len(omf_files) == 201
    assert "{}/m0.omf".format(name) in omf_files

    assert len(odt_files) == 1

    myplot = system.dt.plot("t", "my")
    myplot.figure.savefig("{}/stdprob4-t-my.pdf".format(name))

    assert os.path.isfile("{}/stdprob4-t-my.pdf".format(name))

    t = system.dt["t"].as_matrix()
    my = system.dt["my"].as_matrix()

    assert abs(min(t) - 5e-12) < 1e-20
    assert abs(max(t) - 1e-9) < 1e-20

    # Eye-norm test.
    assert 0.7 < max(my) < 0.8
    assert -0.5 < min(my) < -0.4

    os.system("rm -r {}/".format(name))
